#include <stddef.h>         /* for NULL */
#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */

#include "pt/pt.h"
#include "peri.h"
#include "timer.h"
#include "usbdrv.h"

#define RQ_GET_ECHO_TIME     0
#define RQ_GET_LIGHT         1
#define RQ_BEEP              2

// นิยามมาโคร PT_DELAY ไว้เพื่อจำลองการหน่วงเวลาแบบมัลติทาสกิ้ง
#define PT_DELAY(pt, ms, ts) \
    ts = timer_micro(); \
    PT_WAIT_WHILE(pt, timer_micro()-ts < (ms));
 
struct pt pt_pingultrasonic;
struct pt pt_getultrasonic;

uint32_t dutytime = 0;



uint32_t timer_micro()
{
    return (((TCNT0 - 6) * 4) + (timer_millis() * 1000));
}

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;

    /* declared as static so they stay valid when usbFunctionSetup returns */
    static uint16_t echo_time;
    static uint16_t light_value;

    if (rq->bRequest == RQ_GET_ECHO_TIME)
    {
        echo_time = ((dutytime / 29 )) * 2;
        usbMsgPtr = (uchar*) &echo_time;
        return 2;
    }

    else if (rq->bRequest == RQ_GET_LIGHT)
    {
        light_value = get_light();
        usbMsgPtr = (uchar*) &light_value;
        return sizeof(light_value);
    }
    else if (rq->bRequest == RQ_BEEP)
    {
        
        uint8_t setbeep = rq->wValue.bytes[0];
        beep(setbeep);
        return 0;
    }

    /* default for not implemented requests: return no data back to host */
    return 0;
}

/* ------------------------------------------------------------------------- */

PT_THREAD(pingultrasonic(struct pt* pt))
{
    static uint32_t ts;

    PT_BEGIN(pt);

    for (;;)
    {
        ping_low_ultrasonic();
        _delay_us(2); // 1 = 4 microsec
        ping_high_ultrasonic();
        _delay_us(5);
        ping_low_ultrasonic();
        PT_DELAY(pt,5,ts);
    }

    PT_END(pt);
}

PT_THREAD(getultrasonic(struct pt* pt))
{
    static uint32_t ts;
    static uint32_t t0;

    PT_BEGIN(pt);

    for (;;)
    {
        PT_DELAY(pt,5,ts);
        PT_WAIT_UNTIL(pt,ULTRASONIC_ECHO());
        t0 = timer_micro();
        PT_WAIT_WHILE(pt,ULTRASONIC_ECHO());
        dutytime = timer_micro() - t0;
    }

    PT_END(pt);
}

/* ------------------------------------------------------------------------- */
int main(void)
{
    init_peripheral();

    usbInit();

    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

    timer_init();
    /* enable global interrupts */
    sei();

    PT_INIT(&pt_pingultrasonic);
    PT_INIT(&pt_getultrasonic);

    /* main event loop */
    for(;;)
    {
        usbPoll();
        pingultrasonic(&pt_pingultrasonic);
        getultrasonic(&pt_getultrasonic);
    }

    return 0;
}

/* ------------------------------------------------------------------------- */
